searchData={"items":[{"type":"module","title":"Pigeon","doc":"HTTP2-compliant wrapper for sending iOS and Android push notifications.","ref":"Pigeon.html"},{"type":"module","title":"Getting Started - Pigeon","doc":"Check the module documentation for your push notification service.\n\n  - `Pigeon.ADM` - Amazon Android.\n  - `Pigeon.APNS` - Apple iOS.\n  - `Pigeon.FCM` - Firebase Cloud Messaging v1 API.","ref":"Pigeon.html#module-getting-started"},{"type":"module","title":"Creating Dynamic Runtime Dispatchers - Pigeon","doc":"Pigeon can spin up dynamic dispatchers for a variety of advanced use-cases, such as\nsupporting dozens of dispatcher configurations or custom connection pools.\n\nSee `Pigeon.Dispatcher` for instructions.","ref":"Pigeon.html#module-creating-dynamic-runtime-dispatchers"},{"type":"module","title":"Writing a Custom Dispatcher Adapter - Pigeon","doc":"Want to write a Pigeon adapter for an unsupported push notification service?\n\nSee `Pigeon.Adapter` for instructions.","ref":"Pigeon.html#module-writing-a-custom-dispatcher-adapter"},{"type":"function","title":"Pigeon.default_pool_size/0","doc":"Returns the configured default pool size for Pigeon dispatchers.\nTo customize this value, include the following in your config/config.exs:\n\n    config :pigeon, :default_pool_size, 5","ref":"Pigeon.html#default_pool_size/0"},{"type":"function","title":"Pigeon.json_library/0","doc":"Returns the configured JSON encoding library for Pigeon.\nTo customize the JSON library, include the following in your config/config.exs:\n\n    config :pigeon, :json_library, Jason","ref":"Pigeon.html#json_library/0"},{"type":"function","title":"Pigeon.push/3","doc":"Sends a push notification with given options.","ref":"Pigeon.html#push/3"},{"type":"type","title":"Pigeon.notification/0","doc":"","ref":"Pigeon.html#t:notification/0"},{"type":"type","title":"Pigeon.on_response/0","doc":"Async callback for push notifications response.","ref":"Pigeon.html#t:on_response/0"},{"type":"type","title":"Examples - Pigeon.on_response/0","doc":"handler = fn(%Pigeon.ADM.Notification{response: response}) ->\n      case response do\n        :success ->\n          Logger.debug \"Push successful!\"\n        :unregistered ->\n          Logger.error \"Bad device token!\"\n        _error ->\n          Logger.error \"Some other error happened.\"\n      end\n    end\n\n    n = Pigeon.ADM.Notification.new(\"token\", %{\"message\" => \"test\"})\n    Pigeon.ADM.push(n, on_response: handler)","ref":"Pigeon.html#t:on_response/0-examples"},{"type":"type","title":"Pigeon.push_opts/0","doc":"Options for sending push notifications.\n\n- `:on_response` - Optional async callback triggered on receipt of push.\n  See `t:on_response/0`\n- `:timeout` - Push timeout. Defaults to 5000ms.","ref":"Pigeon.html#t:push_opts/0"},{"type":"behaviour","title":"Pigeon.Adapter","doc":"Adapter behaviour for `Pigeon.Dispatcher` push workers.\n\n`Pigeon.Adapter` closely resembles `GenServer` behaviour, as dispatchers\nare GenServers under the hood.","ref":"Pigeon.Adapter.html"},{"type":"behaviour","title":"Example Adapter - Pigeon.Adapter","doc":"```\ndefmodule Pigeon.Sandbox do\n  import Pigeon.Tasks, only: [process_on_response: 1]\n\n  @behaviour Pigeon.Adapter\n\n  @impl true\n  def init(opts \\ []) do\n    {:ok, opts}\n  end\n\n  @impl true\n  def handle_info(_msg, state) do\n    {:noreply, state}\n  end\n\n  @impl true\n  def handle_push(%{response: nil} = notification, state) do\n    process_on_response(%{notification | response: :success})\n    {:noreply, state}\n  end\n\n  def handle_push(notification, state) do\n    process_on_response(notification)\n    {:noreply, state}\n  end\nend\n```","ref":"Pigeon.Adapter.html#module-example-adapter"},{"type":"callback","title":"Pigeon.Adapter.handle_info/2","doc":"Invoked to handle all other messages.","ref":"Pigeon.Adapter.html#c:handle_info/2"},{"type":"callback","title":"Pigeon.Adapter.handle_push/2","doc":"Invoked to handle push notifications.","ref":"Pigeon.Adapter.html#c:handle_push/2"},{"type":"callback","title":"Pigeon.Adapter.init/1","doc":"Invoked when the server is started.\n\nReturn value should be `{:ok, state}` for the `Pigeon.Dispatcher` state,\nor `{:stop, atom}` if started with invalid configuration options.","ref":"Pigeon.Adapter.html#c:init/1"},{"type":"module","title":"Pigeon.Dispatcher","doc":"Dispatcher worker for push notifications.\n\nIf your push workers are relatively static, it is encouraged to follow the adapter\nguides. For other use cases, such as supporting dynamic configurations, dispatchers\ncan be started and stopped as needed.","ref":"Pigeon.Dispatcher.html"},{"type":"module","title":"Using Dynamic Dispatchers - Pigeon.Dispatcher","doc":"```\n# FCM as an example, but use the relevant options for your push type.\nopts = [\n  adapter: Pigeon.FCM,\n  auth: YourApp.Goth,\n  project_id: \"example-project-123\"\n]\n\n{:ok, pid} = Pigeon.Dispatcher.start_link(opts)\nnotification = Pigeon.FCM.Notification.new({:token, \"regid\"})\n\nPigeon.push(pid, notification)\n```","ref":"Pigeon.Dispatcher.html#module-using-dynamic-dispatchers"},{"type":"module","title":"Loading Configurations from a Database - Pigeon.Dispatcher","doc":"```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      {Goth, name: YourApp.Goth},\n      YourApp.Repo,\n      {Registry, keys: :unique, name: Registry.YourApp}\n    ] ++ push_workers()\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp push_workers do\n    YourApp.Repo.PushApplication\n    |> YourApp.Repo.all()\n    |> Enum.map(&push_spec/1)\n  end\n\n  defp push_spec(%{type: \"apns\"} = config)\n    {Pigeon.Dispatcher, [\n      adapter: Pigeon.APNS,\n      key: config.key,\n      key_identifier: config.key_identifier,\n      team_id: config.team_id,\n      mode: config.mode,\n      name: {:via, Registry, {Registry.YourApp, config.name}}\n    ]}\n  end\n\n  defp push_spec(%{type: \"fcm\"} = config) do\n    {Pigeon.Dispatcher, [\n      adapter: Pigeon.FCM,\n      auth: String.to_existing_atom(config.auth),\n      name: {:via, Registry, {Registry.YourApp, config.name}},\n      project_id: config.project_id\n    ]}\n  end\nend\n```\n\nOnce running, you can send to any of these workers by name.\n\n```\nPigeon.push({:via, Registry, {Registry.YourApp, \"app1\"}}, notification)\n```","ref":"Pigeon.Dispatcher.html#module-loading-configurations-from-a-database"},{"type":"function","title":"Pigeon.Dispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Pigeon.Dispatcher.html#child_spec/1"},{"type":"function","title":"Pigeon.Dispatcher.init/1","doc":"","ref":"Pigeon.Dispatcher.html#init/1"},{"type":"function","title":"Pigeon.Dispatcher.start_link/1","doc":"","ref":"Pigeon.Dispatcher.html#start_link/1"},{"type":"module","title":"Pigeon.Metadata","doc":"Internal push notification metadata.","ref":"Pigeon.Metadata.html"},{"type":"type","title":"Pigeon.Metadata.t/0","doc":"","ref":"Pigeon.Metadata.html#t:t/0"},{"type":"module","title":"Pigeon.Sandbox","doc":"Sandbox adapter for any kind of push notification.\n\nIf the push notification has `response: nil`, it will be marked\nwith `response: :success`. Otherwise it will return the push notification\nexactly as given.","ref":"Pigeon.Sandbox.html"},{"type":"module","title":"Pigeon.ADM","doc":"`Pigeon.Adapter` for ADM (Amazon Android) push notifications.","ref":"Pigeon.ADM.html"},{"type":"module","title":"Getting Started - Pigeon.ADM","doc":"","ref":"Pigeon.ADM.html#module-getting-started"},{"type":"module","title":"Create a dispatcher. - Pigeon.ADM","doc":"```\n# lib/your_app/adm.ex\n\ndefmodule YourApp.ADM do\n  use Pigeon.Dispatcher, otp_app: :your_app\nend\n```","ref":"Pigeon.ADM.html#module-create-a-dispatcher"},{"type":"module","title":"Configure your dispatcher. - Pigeon.ADM","doc":"Configure your `ADM` dispatcher and start it on application boot.\n\n```\n# config.exs\n\nconfig :your_app, YourApp.ADM,\n  adapter: Pigeon.ADM,\n  client_id: \"your_oauth2_client_id_here\",\n  client_secret: \"your_oauth2_client_secret_here\"\n```\n\nAdd it to your supervision tree.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      YourApp.ADM\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nIf preferred, you can include your configuration directly.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      {YourApp.ADM, adm_opts()}\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp adm_opts do\n    [\n      adapter: Pigeon.ADM, \n      client_id: \"client_id\", \n      client_secret: \"secret\"\n    ]\n  end\nend\n```","ref":"Pigeon.ADM.html#module-configure-your-dispatcher"},{"type":"module","title":"Create a notification. - Pigeon.ADM","doc":"```\nmsg = %{ \"body\" => \"your message\" }\nn = Pigeon.ADM.Notification.new(\"your device registration ID\", msg)\n```","ref":"Pigeon.ADM.html#module-create-a-notification"},{"type":"module","title":"Send the notification. - Pigeon.ADM","doc":"```\nYourApp.ADM.push(n)\n```","ref":"Pigeon.ADM.html#module-send-the-notification"},{"type":"module","title":"Handling Push Responses - Pigeon.ADM","doc":"1. Pass an optional anonymous function as your second parameter.\n\n```\ndata = %{ message: \"your message\" }\nn = Pigeon.ADM.Notification.new(\"device registration ID\", data)\nYourApp.ADM.push(n, on_response: fn(x) -> IO.inspect(x) end)\n```\n\n2. Responses return a notification with an updated `:response` key.\n   You could handle responses like so:\n\n```\non_response_handler = fn(x) ->\n  case x.response do\n    :success ->\n      # Push successful\n      :ok\n    :update ->\n      new_reg_id = x.updated_registration_id\n      # Update the registration ID in the database\n    :invalid_registration_id ->\n      # Remove the bad ID from the database\n    :unregistered ->\n      # Remove the bad ID from the database\n    error ->\n      # Handle other errors\n  end\nend\n\ndata = %{ message: \"your message\" }\nn = Pigeon.ADM.Notification.new(\"your registration id\", data)\nPigeon.ADM.push(n, on_response: on_response_handler)\n```","ref":"Pigeon.ADM.html#module-handling-push-responses"},{"type":"module","title":"Error Responses - Pigeon.ADM","doc":"*Taken from [Amazon Device Messaging docs](https://developer.amazon.com/public/apis/engage/device-messaging/tech-docs/06-sending-a-message)*\n\n| Reason                           | Description                      |\n|----------------------------------|----------------------------------|\n| `:invalid_registration_id`       | Invalid Registration Token       |\n| `:invalid_data`                  | Bad format JSON data             |\n| `:invalid_consolidation_key`     | Invalid Consolidation Key        |\n| `:invalid_expiration`            | Invalid expiresAfter value       |\n| `:invalid_checksum`              | Invalid md5 value                |\n| `:invalid_type`                  | Invalid Type header              |\n| `:unregistered`                  | App instance no longer available |\n| `:access_token_expired`          | Expired OAuth access token       |\n| `:message_too_large`             | Data size exceeds 6 KB           |\n| `:max_rate_exceeded`             | See Retry-After response header  |\n| `:unknown_error`                 | Unknown Error                    |","ref":"Pigeon.ADM.html#module-error-responses"},{"type":"module","title":"Pigeon.ADM.Notification","doc":"Defines Amazon ADM notification struct and convenience constructor functions.","ref":"Pigeon.ADM.Notification.html"},{"type":"function","title":"Pigeon.ADM.Notification.new/2","doc":"Creates `ADM.Notification` struct with device registration ID and optional data payload.","ref":"Pigeon.ADM.Notification.html#new/2"},{"type":"function","title":"Examples - Pigeon.ADM.Notification.new/2","doc":"iex> Pigeon.ADM.Notification.new(\"reg ID\")\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"1B2M2Y8AsgTpgAmY7PhCfg==\",\n      payload: %{\"data\" => %{}},\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }\n\n    iex> Pigeon.ADM.Notification.new(\"reg ID\", %{\"message\" => \"your message\"})\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }\n\n    iex> Pigeon.ADM.Notification.new(\"reg ID\", \"not a map\")\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"1B2M2Y8AsgTpgAmY7PhCfg==\",\n      payload: %{\"data\" => %{}},\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#new/2-examples"},{"type":"function","title":"Pigeon.ADM.Notification.put_data/2","doc":"Updates `\"data\"` key on push payload and calculates `md5` hash.","ref":"Pigeon.ADM.Notification.html#put_data/2"},{"type":"function","title":"Examples - Pigeon.ADM.Notification.put_data/2","doc":"iex> n = %Pigeon.ADM.Notification{}\n    iex> Pigeon.ADM.Notification.put_data(n, %{\"message\" => \"your message\"})\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: nil,\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#put_data/2-examples"},{"type":"type","title":"Pigeon.ADM.Notification.error_response/0","doc":"ADM error responses","ref":"Pigeon.ADM.Notification.html#t:error_response/0"},{"type":"type","title":"Pigeon.ADM.Notification.response/0","doc":"ADM push response\n\n- nil - Push has not been sent yet\n- `:success` - Push was successfully sent\n- `t:error_response/0` - Push attempted but server responded\n  with error\n- `:timeout` - Internal error. Push did not reach ADM servers","ref":"Pigeon.ADM.Notification.html#t:response/0"},{"type":"type","title":"Pigeon.ADM.Notification.t/0","doc":"ADM notification","ref":"Pigeon.ADM.Notification.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.ADM.Notification.t/0","doc":"%Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      expires_after: 604_800,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: \"reg ID\",\n      response: nil, # Set on push response\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#t:t/0-examples"},{"type":"module","title":"Pigeon.APNS","doc":"`Pigeon.Adapter` for Apple Push Notification Service (APNS) push notifications.","ref":"Pigeon.APNS.html"},{"type":"module","title":"Getting Started - Pigeon.APNS","doc":"","ref":"Pigeon.APNS.html#module-getting-started"},{"type":"module","title":"Create a dispatcher. - Pigeon.APNS","doc":"```\n# lib/your_app/apns.ex\n\ndefmodule YourApp.APNS do\n  use Pigeon.Dispatcher, otp_app: :your_app\nend\n```","ref":"Pigeon.APNS.html#module-create-a-dispatcher"},{"type":"module","title":"Configure your dispatcher. - Pigeon.APNS","doc":"Configure your `APNS` dispatcher and start it on application boot.\n\n```\n# config.exs\n\nconfig :your_app, YourApp.APNS,\n  adapter: Pigeon.APNS,\n  cert: File.read!(\"cert.pem\"),\n  key: File.read!(\"key_unencrypted.pem\"),\n  mode: :dev\n\n# Or for token based authentication:\n\nconfig :your_app, YourApp.APNS,\n  adapter: Pigeon.APNS,\n  key: File.read!(\"AuthKey.p8\"),\n  key_identifier: \"ABC1234567\",\n  mode: :dev,\n  team_id: \"DEF8901234\"\n```\n\nAdd it to your supervision tree.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      YourApp.APNS\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nIf preferred, you can include your configuration directly.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      {YourApp.APNS, apns_opts()}\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp apns_opts do\n    [\n      adapter: Pigeon.APNS,\n      cert: File.read!(\"cert.pem\"),\n      key: File.read!(\"key_unencrypted.pem\"),\n      mode: :dev\n    ]\n  end\nend\n```","ref":"Pigeon.APNS.html#module-configure-your-dispatcher"},{"type":"module","title":"Create a notification. - Pigeon.APNS","doc":"```\nn = Pigeon.APNS.Notification.new(\"your message\", \"your device token\", \"your push topic\")\n```\n\n> #### Note {: .info}\n>\n> Note: Your push topic is generally the app's bundle identifier.","ref":"Pigeon.APNS.html#module-create-a-notification"},{"type":"module","title":"Send the notification. - Pigeon.APNS","doc":"Pushes are synchronous and return the notification with an updated `:response` key.\n\n```\nYourApp.APNS.push(n)\n```","ref":"Pigeon.APNS.html#module-send-the-notification"},{"type":"module","title":"Configuration Options - Pigeon.APNS","doc":"#### Certificate Authentication\n\n- `:cert` - Push certificate. Must be the full-text string of the file contents.\n- `:key` - Push private key. Must be the full-text string of the file contents.\n\n#### Token Authentication\n\n- `:key` - JWT private key. Must be the full-text string of the file contents.\n- `:key_identifier` - A 10-character key identifier (kid) key, obtained from\n  your developer account.\n- `:team_id` - Your 10-character Team ID, obtained from your developer account.\n\n#### Shared Options\n\n- `:mode` - If set to `:dev` or `:prod`, will set the appropriate `:uri`.\n- `:ping_period` - Interval between server pings. Necessary to keep long\n  running APNS connections alive. Defaults to 10 minutes.\n- `:port` - Push server port. Can be any value, but APNS only accepts\n  `443` and `2197`.\n- `:uri` - Push server uri. If set, overrides uri defined by `:mode`.\n  Useful for test environments.","ref":"Pigeon.APNS.html#module-configuration-options"},{"type":"module","title":"Generating Your Certificate and Key .pem - Pigeon.APNS","doc":"1. In Keychain Access, right-click your push certificate and select _\"Export...\"_\n2. Export the certificate as `cert.p12`\n3. Click the dropdown arrow next to the certificate, right-click the private\n   key and select _\"Export...\"_\n4. Export the private key as `key.p12`\n5. From a shell, convert the certificate.\n\n```\nopenssl pkcs12 -legacy -clcerts -nokeys -out cert.pem -in cert.p12\n```\n\n6. Convert the key. Be sure to set a PEM pass phrase here. The pass phrase must be 4 or \n   more characters in length or this will not work. You will need that pass phrase added \n   here in order to remove it in the next step.\n\n```\nopenssl pkcs12 -legacy -nocerts -out key.pem -in key.p12\n```\n\n7. Remove the PEM pass phrase from the key.\n\n```\nopenssl rsa -in key.pem -out key_unencrypted.pem\n```\n\n8. `cert.pem` and `key_unencrypted.pem` can now be used in your configuration.","ref":"Pigeon.APNS.html#module-generating-your-certificate-and-key-pem"},{"type":"module","title":"Pigeon.APNS.Notification","doc":"Defines APNS notification struct and constructor functions.","ref":"Pigeon.APNS.Notification.html"},{"type":"function","title":"Pigeon.APNS.Notification.new/3","doc":"Returns an `APNS.Notification` struct with given message, device token, and\ntopic (optional).\n\nPush payload is constructed in the form of `%{\"aps\" => %{\"alert\" => msg}}`","ref":"Pigeon.APNS.Notification.html#new/3"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.new/3","doc":"iex> Pigeon.APNS.Notification.new(\"push message\", \"device token\")\n    %Pigeon.APNS.Notification{\n      __meta__: %Pigeon.Metadata{},\n      device_token: \"device token\",\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#new/3-examples"},{"type":"function","title":"Pigeon.APNS.Notification.new/4","doc":"Returns an `APNS.Notification` struct with given message, device token,\ntopic, and message ID.\n\nPush payload is constructed in the form of `%{\"aps\" => %{\"alert\" => msg}}`","ref":"Pigeon.APNS.Notification.html#new/4"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.new/4","doc":"iex> Pigeon.APNS.Notification.new(\"push message\", \"device token\", \"topic\", \"id_1234\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: \"device token\",\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: \"id_1234\",\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n      topic: \"topic\"\n    }","ref":"Pigeon.APNS.Notification.html#new/4-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_alert/2","doc":"Updates `\"alert\"` key in push payload.\n\nThis is the alert message displayed on the device.","ref":"Pigeon.APNS.Notification.html#put_alert/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_alert/2","doc":"iex> Pigeon.APNS.Notification.put_alert(%Pigeon.APNS.Notification{}, \"push message\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}}\n    }","ref":"Pigeon.APNS.Notification.html#put_alert/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_badge/2","doc":"Updates `\"badge\"` key in push payload.\n\nThis is the badge number displayed on the application.","ref":"Pigeon.APNS.Notification.html#put_badge/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_badge/2","doc":"iex> Pigeon.APNS.Notification.put_badge(%Pigeon.APNS.Notification{}, 5)\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"badge\" => 5}}\n    }","ref":"Pigeon.APNS.Notification.html#put_badge/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_category/2","doc":"Updates `\"category\"` key in push payload.","ref":"Pigeon.APNS.Notification.html#put_category/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_category/2","doc":"iex> Pigeon.APNS.Notification.put_category(%Pigeon.APNS.Notification{}, \"category\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"category\" => \"category\"}}\n    }","ref":"Pigeon.APNS.Notification.html#put_category/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_content_available/1","doc":"Sets `\"content-available\"` flag in push payload.\n\nUsed for silent notifications. When set, ensure `alert`, `badge`, and `sound` keys\nare not configured.","ref":"Pigeon.APNS.Notification.html#put_content_available/1"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_content_available/1","doc":"iex> Pigeon.APNS.Notification.put_content_available(%Pigeon.APNS.Notification{})\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"content-available\" => 1}}\n    }","ref":"Pigeon.APNS.Notification.html#put_content_available/1-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_custom/2","doc":"Puts custom data in push payload.","ref":"Pigeon.APNS.Notification.html#put_custom/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_custom/2","doc":"iex> n = Pigeon.APNS.Notification.new(\"test message\", \"device token\")\n    iex> Pigeon.APNS.Notification.put_custom(n, %{\"custom-key\" => 1234})\n    %Pigeon.APNS.Notification{\n      device_token: \"device token\",\n      payload: %{\"aps\" => %{\"alert\" => \"test message\"}, \"custom-key\" => 1234}\n    }","ref":"Pigeon.APNS.Notification.html#put_custom/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_interruption_level/2","doc":"Updates `\"interruption-level\"` key in push payload.\n\nUsed for managing time sensitive notifications","ref":"Pigeon.APNS.Notification.html#put_interruption_level/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_interruption_level/2","doc":"iex> Pigeon.APNS.Notification.put_interruption_level(%Pigeon.APNS.Notification{}, \"time-sensitive\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"interruption-level\" => \"time-sensitive\"}}\n    }","ref":"Pigeon.APNS.Notification.html#put_interruption_level/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_mutable_content/1","doc":"Sets `\"mutable-content\"` flag in push payload.\n\nUsed for notification service extensions (such as displaying custom media).","ref":"Pigeon.APNS.Notification.html#put_mutable_content/1"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_mutable_content/1","doc":"iex> Pigeon.APNS.Notification.put_mutable_content(%Pigeon.APNS.Notification{})\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"mutable-content\" => 1}}\n    }","ref":"Pigeon.APNS.Notification.html#put_mutable_content/1-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_sound/2","doc":"Updates `\"sound\"` key in push payload.\n\nUsed for custom notification sounds. Value should\nbe the name of the custom sound file in the application's binary.","ref":"Pigeon.APNS.Notification.html#put_sound/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_sound/2","doc":"iex> Pigeon.APNS.Notification.put_sound(%Pigeon.APNS.Notification{}, \"custom.aiff\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"sound\" => \"custom.aiff\"}}\n    }\n\n    iex> Pigeon.APNS.Notification.put_sound(%Pigeon.APNS.Notification{}, %{\n    ...>   \"critical\" => 1,\n    ...>   \"sound\" => \"default\",\n    ...>   \"volume\" => 1.0\n    ...> })\n    %Pigeon.APNS.Notification{\n      payload: %{\n        \"aps\" => %{\n          \"sound\" => %{\n            \"critical\" => 1,\n            \"sound\" => \"default\",\n            \"volume\" => 1.0\n          }\n        }\n      }\n    }","ref":"Pigeon.APNS.Notification.html#put_sound/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_target_content_id/2","doc":"Updates `\"target-content-id\"` key in push payload.\n\nUsed for bringing a specific window forward.","ref":"Pigeon.APNS.Notification.html#put_target_content_id/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_target_content_id/2","doc":"iex> Pigeon.APNS.Notification.put_target_content_id(%Pigeon.APNS.Notification{}, \"example\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"target-content-id\" => \"example\"}}\n    }","ref":"Pigeon.APNS.Notification.html#put_target_content_id/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_thread_id/2","doc":"Updates `\"thread-id\"` key in push payload.\n\nUsed for grouping related notifications.","ref":"Pigeon.APNS.Notification.html#put_thread_id/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_thread_id/2","doc":"iex> Pigeon.APNS.Notification.put_thread_id(%Pigeon.APNS.Notification{}, \"example\")\n    %Pigeon.APNS.Notification{\n      payload: %{\"aps\" => %{\"thread-id\" => \"example\"}}\n    }","ref":"Pigeon.APNS.Notification.html#put_thread_id/2-examples"},{"type":"type","title":"Pigeon.APNS.Notification.error_response/0","doc":"","ref":"Pigeon.APNS.Notification.html#t:error_response/0"},{"type":"type","title":"Pigeon.APNS.Notification.response/0","doc":"APNS push response\n\n- nil - Push has not been sent yet.\n- `:success` - Push was successfully sent.\n- `t:Pigeon.APNS.Error.error_response/0` - Push attempted but\n   server responded with error.\n- `:timeout` - Internal error. Push did not reach APNS servers.","ref":"Pigeon.APNS.Notification.html#t:response/0"},{"type":"type","title":"Pigeon.APNS.Notification.t/0","doc":"APNS notification","ref":"Pigeon.APNS.Notification.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.APNS.Notification.t/0","doc":"%Pigeon.APNS.Notification{\n        __meta__: %Pigeon.Metadata{on_response: nil},\n        collapse_id: nil,\n        device_token: \"device token\",\n        expiration: nil,\n        id: nil, # Set on push response if nil\n        payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n        priority: nil,\n        push_type: \"alert\",\n        response: nil, # Set on push response\n        topic: \"com.example.YourApp\"\n    }","ref":"Pigeon.APNS.Notification.html#t:t/0-examples"},{"type":"module","title":"Pigeon.FCM","doc":"`Pigeon.Adapter` for Firebase Cloud Messaging (FCM) push notifications.","ref":"Pigeon.FCM.html"},{"type":"module","title":"Getting Started - Pigeon.FCM","doc":"","ref":"Pigeon.FCM.html#module-getting-started"},{"type":"module","title":"Create a dispatcher. - Pigeon.FCM","doc":"```\n  # lib/your_app/fcm.ex\n\n  defmodule YourApp.FCM do\n    use Pigeon.Dispatcher, otp_app: :your_app\n  end\n  ```","ref":"Pigeon.FCM.html#module-create-a-dispatcher"},{"type":"module","title":"Install and configure Goth. - Pigeon.FCM","doc":"Install and configure [`goth`](https://hexdocs.pm/goth/1.4.3/readme.html#installation)\nif you haven't already. `Pigeon.FCM` requires it for token authentication.","ref":"Pigeon.FCM.html#module-install-and-configure-goth"},{"type":"module","title":"Configure your dispatcher. - Pigeon.FCM","doc":"Configure your `FCM` dispatcher and start it on application boot.\n\n```\n# config.exs\n\nconfig :your_app, YourApp.FCM,\n  adapter: Pigeon.FCM,\n  auth: YourApp.Goth, # Your Goth worker configured in the previous step.\n  project_id: \"example-project-123\"\n```\n\nAdd it to your supervision tree.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      {Goth, name: YourApp.Goth},\n      YourApp.FCM\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nIf preferred, you can include your configuration directly.\n\n```\ndefmodule YourApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @doc false\n  def start(_type, _args) do\n    children = [\n      {Goth, name: YourApp.Goth},\n      {YourApp.FCM, fcm_opts()}\n    ]\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  defp fcm_opts do\n    [\n      adapter: Pigeon.FCM,\n      auth: YourApp.Goth,\n      project_id: \"example-project-123\"\n    ]\n  end\nend\n```","ref":"Pigeon.FCM.html#module-configure-your-dispatcher"},{"type":"module","title":"Create a notification. - Pigeon.FCM","doc":"```\nn = Pigeon.FCM.Notification.new({:token, \"reg ID\"}, %{\"body\" => \"test message\"})\n```","ref":"Pigeon.FCM.html#module-create-a-notification"},{"type":"module","title":"Send the notification. - Pigeon.FCM","doc":"On successful response, `:name` will be set to the name returned from the FCM\nAPI and `:response` will be `:success`. If there was an error, `:error` will\ncontain a JSON map of the response and `:response` will be an atomized version\nof the error type.\n\n```\nYourApp.FCM.push(n)\n```","ref":"Pigeon.FCM.html#module-send-the-notification"},{"type":"module","title":"Customizing Goth - Pigeon.FCM","doc":"You can use any of the configuration options (e.g. `:source`) for Goth. Check out the \ndocumentation of [`Goth.start_link/1`](https://hexdocs.pm/goth/Goth.html#start_link/1) \nfor more details.","ref":"Pigeon.FCM.html#module-customizing-goth"},{"type":"module","title":"Pigeon.FCM.Notification","doc":"Defines FCM notification struct and convenience constructor functions.","ref":"Pigeon.FCM.Notification.html"},{"type":"function","title":"Pigeon.FCM.Notification.new/3","doc":"Creates `FCM.Notification` struct with given target and optional\nnotification and data payloads.","ref":"Pigeon.FCM.Notification.html#new/3"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.new/3","doc":"iex> Pigeon.FCM.Notification.new({:token, \"reg ID\"})\n    %Pigeon.FCM.Notification{\n      data: nil,\n      notification: nil,\n      target: {:token, \"reg ID\"}\n    }\n\n    iex> Pigeon.FCM.Notification.new({:topic, \"example\"})\n    %Pigeon.FCM.Notification{\n      data: nil,\n      notification: nil,\n      target: {:topic, \"example\"}\n    }\n\n    iex> Pigeon.FCM.Notification.new({:token, \"reg ID\"}, %{\"body\" => \"test message\"})\n    %Pigeon.FCM.Notification{\n      data: nil,\n      notification: %{\"body\" => \"test message\"},\n      target: {:token, \"reg ID\"}\n    }\n\n    iex> Pigeon.FCM.Notification.new({:token, \"reg ID\"}, %{\"body\" => \"test message\"},\n    ...> %{\"key\" => \"value\"})\n    %Pigeon.FCM.Notification{\n      data: %{\"key\" => \"value\"},\n      notification: %{\"body\" => \"test message\"},\n      target: {:token, \"reg ID\"}\n    }","ref":"Pigeon.FCM.Notification.html#new/3-examples"},{"type":"type","title":"Pigeon.FCM.Notification.error_response/0","doc":"","ref":"Pigeon.FCM.Notification.html#t:error_response/0"},{"type":"type","title":"Pigeon.FCM.Notification.t/0","doc":"","ref":"Pigeon.FCM.Notification.html#t:t/0"},{"type":"type","title":"Pigeon.FCM.Notification.target/0","doc":"FCM notification target. Must be one of the following:\n\n- `{:token, \"string\"}` - Registration token to send a message to.\n- `{:topic, \"string\"}` - Topic name to send a message to, e.g. \"weather\". \n  Note: \"/topics/\" prefix should not be provided.\n- `{:condition, \"string\"}` - Condition to send a message to, e.g. \"'foo' \n  in topics && 'bar' in topics\".","ref":"Pigeon.FCM.Notification.html#t:target/0"},{"type":"exception","title":"Pigeon.ConfigError","doc":"","ref":"Pigeon.ConfigError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.0.0-rc.3 - Changelog","doc":"","ref":"changelog.html#v2-0-0-rc-3"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- `Pigeon.LegacyFCM` has been removed entirely and migrated to [pigeon_legacy_fcm](https://github.com/codedge-llc/pigeon-legacy-fcm) package.\n- FCM `service_account_json` config option has been removed and replaced with `:auth`, a [Goth](https://github.com/peburrows/goth) configuration.\n  See `Pigeon.FCM` documentation for setup and [#235](https://github.com/codedge-llc/pigeon/pull/235) for more details.\n\n**Changed**\n\n- Bump `goth` dependency to `~> 1.4.3`. ([#252](https://github.com/codedge-llc/pigeon/pull/252))\n\n**Fixed**\n\n- `DispatcherWorker` missing a clause for `{:stop, reason}` in the handle_info function.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v2.0.0-rc.2 - Changelog","doc":"**Fixed**\n\n- Resolve APNS `:too_many_provider_token_updates` by moving token generation into\n  `APNS.Token` ([#227](https://github.com/codedge-llc/pigeon/pull/227)).\n- Support HTTPoison 2.0. ([#236](https://github.com/codedge-llc/pigeon/pull/236))\n- Improve handling of FCM error responses. ([#245](https://github.com/codedge-llc/pigeon/pull/245))\n- Fix `DispatcherWorker` missing a clause for `{:stop, reason}` in the init function.\n- `APNS.Config` keys now decode properly for PEMs generated with OpenSSL 3. ([#248](https://github.com/codedge-llc/pigeon/pull/248))\n- Add `ExpiredToken` as APNS error response. ([#240](https://github.com/codedge-llc/pigeon/pull/240))\n- Better handling of APNS token lifecycle between `:dev`/`:prod` environments with the same key identifier. ([#239](https://github.com/codedge-llc/pigeon/pull/239))","ref":"changelog.html#v2-0-0-rc-2"},{"type":"extras","title":"v2.0.0-rc.1 - Changelog","doc":"**Fixed**\n\n- Reset stream ID when connection is closed. ([#216](https://github.com/codedge-llc/pigeon/pull/217))\n\n**Changed**\n\n- Bump `goth` dependency to `~> 1.3.0`. ([#224](https://github.com/codedge-llc/pigeon/pull/224))\n- Bump minimum supported Elixir version to `1.7`.","ref":"changelog.html#v2-0-0-rc-1"},{"type":"extras","title":"v2.0.0-rc.0 - Changelog","doc":"**Changed**\n\n- Default JSON library set to Jason.\n  ([#182](https://github.com/codedge-llc/pigeon/pull/182))\n- Pigeon application module moved from `Pigeon` to `Pigeon.Application`.\n  ([#183](https://github.com/codedge-llc/pigeon/pull/183))\n- Kadabra bumped to v0.5.0, and now a required dependency.\n  ([#184](https://github.com/codedge-llc/pigeon/pull/184))\n- Sending a list of pushes synchronously now actually sends them one at a time. For production\n  workloads, using the async `:on_response` callback is strongly suggested.\n\n**Fixed**\n\n- Removed Elixir 1.11 compile warnings.\n  ([#184](https://github.com/codedge-llc/pigeon/pull/184))\n\n**Removed**\n\n- `:certfile` and `:keyfile` are no longer valid options for APNS configurations.\n  Instead, read the file before loading (e.g. `cert: File.read!(\"cert.pem\")`)\n  ([#183](https://github.com/codedge-llc/pigeon/pull/183))\n- `:debug_log` removed.","ref":"changelog.html#v2-0-0-rc-0"},{"type":"extras","title":"v1.6.0 - Changelog","doc":"**Added**\n\n- JSON library made configurable. For backwards compatibility, Poison is still a required dependency.\n  Override in your `config.exs`.\n\n  ```\n  config :pigeon, json_library: Jason\n  ```\n\n**Fixed**\n\n- Handle FCM single message_id's on topic pushes.","ref":"changelog.html#v1-6-0"},{"type":"extras","title":"v1.5.1 - Changelog","doc":"- Added APNS InvalidPushType error ([#172](https://github.com/codedge-llc/pigeon/pull/172)).\n- Fixed various typespecs ([#170](https://github.com/codedge-llc/pigeon/pull/170)).","ref":"changelog.html#v1-5-1"},{"type":"extras","title":"v1.5.0 - Changelog","doc":"- Bumped minimum Elixir version to 1.6\n- Raise `Pigeon.ConfigError` when booting invalid config structs.\n  See below for validated keys and error types.\n- `APNS.JWTConfig` now validates key p8 content before connecting.\n- Relaxed `gen_stage` dependency to allow `~> 1.0`\n\nValidated config keys:\n\n- `ADM.Config` - `:client_id`, `:client_secret`\n- `APNS.Config` - `:cert`, `:key`\n- `APNS.JWTConfig` - `:team_id`, `:key`, `:key_identifier`\n- `FCM.Config` - `:key`\n\nPossible error values:\n\n- `{:error, {:invalid, value}}`\n- `{:error, {:nofile, value}}`","ref":"changelog.html#v1-5-0"},{"type":"extras","title":"v1.4.0 - Changelog","doc":"- `apns-push-type` header support for iOS 13. An additional `:push_type` key has been\n  added to the `APNS.Notification` struct.","ref":"changelog.html#v1-4-0"},{"type":"extras","title":"v1.3.2 - Changelog","doc":"- Document workers configuration for run-time configuration of push workers.\n- Modify run-time configuration of push workers so that multiple (or no)\n  workers may be returned by the startup configuration.","ref":"changelog.html#v1-3-2"},{"type":"extras","title":"v1.3.1 - Changelog","doc":"- Joken dependency bumped to 2.1","ref":"changelog.html#v1-3-1"},{"type":"extras","title":"v1.3.0 - Changelog","doc":"- Support for FCM `content_available`, `mutable_content`, and `condition` keys\n- Set `priority` of APNS notifications\n- Joken dependency bumped to 2.0.1","ref":"changelog.html#v1-3-0"},{"type":"extras","title":"v1.2.4 - Changelog","doc":"- Fixed ADM handling of connection timeouts","ref":"changelog.html#v1-2-4"},{"type":"extras","title":"v1.2.3 - Changelog","doc":"- Fixed APNS, FCM and ADM error response parse crashes. Error responses not\n  listed in the documentation are returned as `:unknown_error`","ref":"changelog.html#v1-2-3"},{"type":"extras","title":"v1.2.2 - Changelog","doc":"- Fixed APNS handling of notification `expiration`\n- Added APNS support for `collapse_id`","ref":"changelog.html#v1-2-2"},{"type":"extras","title":"v1.2.1 - Changelog","doc":"- FCM notifications can now handle `time_to_live`, `collapse_key`, `restricted_package_name`\n  and `dry_run` keys","ref":"changelog.html#v1-2-1"},{"type":"extras","title":"v1.2.0 - Changelog","doc":"- Support for APNS JWT configuration\n- Bump `kadabra` dependency to `v0.4.2`","ref":"changelog.html#v1-2-0"},{"type":"extras","title":"v1.1.6 - Changelog","doc":"- Relax `gen_stage` dependency to `~> 0.12`\n- Bump `kadabra` dependency to `v0.3.7`","ref":"changelog.html#v1-1-6"},{"type":"extras","title":"v1.1.5 - Changelog","doc":"- Fix: relax `httpoison` dependency to allow `0.x` or `1.0`","ref":"changelog.html#v1-1-5"},{"type":"extras","title":"v1.1.4 - Changelog","doc":"- Fix: `:on_response` callbacks spawned as supervised task instead of running\n  in the `Worker` process\n- Fix: ADM token refresh failure returns updated notification instead of\n  error tuple","ref":"changelog.html#v1-1-4"},{"type":"extras","title":"v1.1.3 - Changelog","doc":"- More robust FCM/APNS backpressure\n- Bumped minimum Kadabra version to `v0.3.6`","ref":"changelog.html#v1-1-3"},{"type":"extras","title":"v1.1.2 - Changelog","doc":"- Auto-restart connections if max stream ID is reached\n- FCM/APNS Workers now use GenStage to queue pending pushes\n- Bumped minimum Kadabra version to `v0.3.5`","ref":"changelog.html#v1-1-2"},{"type":"extras","title":"v1.1.1 - Changelog","doc":"- Bumped minimum Kadabra version to `v0.3.4`","ref":"changelog.html#v1-1-1"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"- Minimum requirements now Elixir v1.4 and OTP 19.2 (Kadabra bumped\n  to `v0.3.0`)\n- Startup worker configs. Create a functions that return config\n  structs and specify them your `config.exs` with\n\n```elixir\nconfig :pigeon, workers: [\n  {YourApp.Pigeon, :apns_config},\n  {YourApp.Pigeon, :fcm_config},\n  {YourApp.Pigeon, :adm_config},\n  ...\n]\n```\n\n**APNS**\n\n- `APNS.Config.config/1` renamed to `APNS.Config.new/1`\n- `APNS.push/2` tagged tuples done away with in favor of a `:response` key on\n  the notification.\n- Override push server endpoint with `:uri` option in `APNS.Config.new/1`\n- `:use_2197` renamed to `:port`\n- `:uri` config option for overriding push server endpoint\n- `:reconnect` now false by default\n\n**FCM**\n\n- `NotificationResponse` done away with in favor of a `:response` key\n  on `Notification`\n- Override push server endpoint with `:uri` and `:port` options\n  in `FCM.Config.new/1`\n- `:uri` and `:port` config options for overriding push server endpoint\n\n**ADM**\n\n- `ADM.Config.config/1` renamed to `ADM.Config.new/1`\n- `ADM.push/2` tagged tuples done away with in favor of a `:response` key on\n  the notification.\n- `ADM.start_connection/1` and `ADM.stop_connection/1` added","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"v1.0.4 - Changelog","doc":"- Fix: removed connection pinging from FCM.Worker (`:ping_period` option\n  left in FCM config to not break API)","ref":"changelog.html#v1-0-4"},{"type":"extras","title":"v1.0.3 - Changelog","doc":"- Fixed proper handling of large FCM push batches","ref":"changelog.html#v1-0-3"},{"type":"extras","title":"v1.0.2 - Changelog","doc":"- Fixed FCM infinite `GOAWAY session_timed_out` loop","ref":"changelog.html#v1-0-2"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"- Configurable `:ping_period` for FCM connections","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"- GCM migrated to FCM API (http2)\n- `GCM` modules renamed to `FCM`\n- Set priority of FCM notifications\n- `APNSWorker` and `ADMWorker` renamed to `APNS.Worker` and `ADM.Worker`\n- Disable auto-reconnect for APNS workers with `reconnect: false`\n- Removed Chatterbox http2 client adapter","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.13.0 - Changelog","doc":"- Configurable `:ping_period` for APNS connections","ref":"changelog.html#v0-13-0"},{"type":"extras","title":"v0.12.1 - Changelog","doc":"- Various `chatterbox` client adapter fixes","ref":"changelog.html#v0-12-1"},{"type":"extras","title":"V0.12.0 - Changelog","doc":"- Configurable `Pigeon.Http2.Client`. Currently supports `kadabra`\n  and `chatterbox`\n- `kadabra` bumped to `v0.2.0`","ref":"changelog.html#v0-12-0"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"- APNS workers can be started and referenced with pids and/or atom names\n- Fix: Push `:name` option renamed to `:to`\n- Fix: GCM/ADM async pushes now use `spawn/1` instead of `Task.async/1`","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"- Fix: cleaned up Elixir v1.4 warnings","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"- Fix: poison dependency version made optionally `~> 2.0 or ~> 3.0`","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"- Fix: kadabra not started","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"- Migrated HTTP/2 client from `chatterbox` to `kadabra`\n- Support for ADM (Amazon Android) push\n- APNS pushes are now synchronous by default. For async pushes use the\n  new `on_response` option. GCM and ADM will have it in the next major release.\n- Bulk APNS pushing\n- Handling of multiple APNS worker connections with different configs\n- Re-implemented APNS socket pings to keep connections open\n- `:invalid_jSON` corrected to `:invalid_json`","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"- Fixed GCM error response atom conversion","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"- Fixed :eaddrinuse error when restarting Pigeon too quickly with\n  :apns_2197 enabled","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"- APNS topic made optional\n- APNS `put_mutable_content` helper function added\n- GCM can be configured on a per-push basis\n- Updated to use Macro.underscore","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"- Implemented Chatterbox as APNS HTTP2 client\n- APNS server responses now caught asynchronously\n- GCM support for `notification` and `data` payload keys\n  (`Pigeon.GCM.Notification.new` API changes)","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"- APNS cert/key configs can now either be a file path, full-text string,\n  or `{:your_app, \"path/to/file.pem\"}` (which looks in the `/priv` directory\n  of your app)\n- Fixed APNSWorker crash on `:ssl.send/2` timeout\n- Better error-handling for invalid APNS configs","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"- `Pigeon.APNS.Notification.new/3` returns `%Pigeon.APNS.Notification{}` struct\n- Configure APNS to use SSL port 2197 with `apns_2197: true` in\n  your `config.exs`\n- Error feedback symbols converted from `CamelCase` to `snake_case`\n- APNS expiration values supported with `expiration` key in\n  `%Pigeon.APNS.Notification{}`","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"- Fixed bug where APNSWorker would hang up if SSL connection failed. Now\n  retries the connection twice more before gracefully shutting down.","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"- GCM error responses return proper chunk of regstration IDs","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- `Pigeon.GCM.Notification.new/2` returns `%Pigeon.GCM.Notification{}` struct\n- Multiple registration IDs allowed in `Pigeon.GCM.push/2`\n- Remove `:gcm_worker`","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"License","doc":"Copyright (c) 2015-2024 Codedge LLC (https://www.codedge.io/)\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}